/* $Source: /usr/local/cvs/snarf/http.c,v $
 * $Id: http.c,v 1.9 1998/02/12 01:31:04 xach Exp $
 */
#include "config.h"
#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/socket.h>
#include "url.h"
#include "http.h"
#include "option.h"
#include "snarf.h"
#include "misc.h"
#ifdef HAVE_SYS_FILE_H
# include <sys/file.h>
#endif
#ifdef HAVE_SYS_FCNTL_H
# include <sys/fcntl.h>
#endif
#include "option.h"

#define MAXLEN 4096

static char rangeheaders[256];
static char rangeackstring[256];

int http_get(struct url u, char *outfile)
{
  struct url proxy;
  char line[MAXLEN], status[MAXLEN];
  char token[MAXLEN];
  char *request = NULL;
  int mysock;
  int output;
  int in_header;
  int open_flags;
  int rangeacknowledged;
  off_t file_offset;

  in_header = 1;

  if(resume) {
    file_offset = getsize(outfile);
    sprintf(rangeheaders, "Range: bytes=%ld-\r\n", (long int )file_offset);
    sprintf(rangeackstring, "bytes %ld-", (long int )file_offset);
  }

  if(get_proxy(&proxy)){
    if(!(mysock = tcp_connect(proxy)))
      return(0);
    get_proxy_request(u, request);
  } else {

    if(!(mysock = tcp_connect(u)))
      return(0);
    request = strdup(get_http_request(u));
  }
  
  send(mysock, request, strlen(request), 0);

  /* hmm...let's do a bit of status checking by reading the numeric
     response code from the server */
  readline(mysock, line);

  if(verbose)
    fprintf(stderr, "%s", line);

  strcpy(status, line);

  if(!zmatch("HTTP", line)){
    in_header = 0;
  }
    
  if(in_header) {
    strtok(line, " ");
    strcpy(token, (char *)strtok(NULL, " "));
    if(token[0] == '4' || token[0] == '5'){
      fprintf(stderr, "http error from server: %s", status);
      return(0);
    }
  }
  
  rangeacknowledged = 0;
  /* skip that inconvenient http header */
  while(in_header) {
    if(!readline(mysock, line)) {
      in_header = 0;
      break;
    }

    if(resume &&
       !strncasecmp(line, "Content-Range:", sizeof("Content-Range:") - 1)
       && strstr(line, rangeackstring))
      rangeacknowledged = 1;

    if(line[0] == '\n' || line[1] == '\n'){
      in_header = 0;

      if(verbose) 
	fprintf(stderr, "\n");

      break;
    }

    if(verbose)
      fprintf(stderr, "%s", line);
  }

  if(resume && !rangeacknowledged) {
    fprintf(stderr, "server doesn't seem to support resume; try again " 
	    "without -r\n");
    return(0);
  }

  open_flags = O_CREAT|O_WRONLY;
  if(resume)
    open_flags |= O_APPEND;
  else
    open_flags |= O_TRUNC;

  if(strcmp(outfile, "-") == 0)
    output = 1;
  else 
    if((output = open(outfile, open_flags, 0644)) < 0) {
      perror(outfile);
      return(0);
    }

  dump_data(mysock, output);
  
  return(1);
}

int get_proxy(struct url *u)
{
  char string[MAXLEN];

  if((char *)getenv("SNARF_PROXY") == NULL)
    return(0);
  strncpy(string, (char *)getenv("SNARF_PROXY"), MAXLEN);
  *u = parse_url(string);

  return(1);
}


int get_proxy_request(struct url u, char *request)
{
  char tmp[MAXLEN];
  request[0] = '\0';

  sprintf(tmp, "GET %s://%s:%d%s%s HTTP/1.0\r\n"
	  "User-agent: snarf %s\r\n\r\n", 
	  u.u_service, u.u_host, u.u_port, u.u_path, u.u_file,
	  VERSION);
  strncpy(request, tmp, MAXLEN);
  return(1);
}

char *get_http_request(struct url u)
{
  char tmp[MAXLEN];
  char *request = NULL;
  char *pwstring;

  /* size of the two strings, plus ':' and trailing null */

  pwstring = (char *) malloc( strlen(u.u_user) +
			      strlen(u.u_pass) +
			      2 );

  sprintf(pwstring, "%s:%s", u.u_user, u.u_pass);

  sprintf(tmp, "GET %s%s HTTP/1.%c\r\n"
  	  "Host: %s\r\n"
	  "User-agent: snarf/%s\r\n"
	  "Authorization: Basic %s\r\n",
	  u.u_path, u.u_file, 
	  (resume ? '1' : '0'), 
	  u.u_host, VERSION,
	  base64(pwstring, strlen(pwstring)));
  if(resume)
    strcat(tmp, rangeheaders);

  strcat(tmp, "\r\n");

  request = strdup(tmp);
  return(request);
}
